/*
 *  Copyright (c) 2006  Peter Kliem (Peter.Kliem@jaret.de)
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 */
package de.jaret.ppc.wm.model;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;
/**
 * Preference properties.
 * 
 * @author kliem
 * @version $Id: Preferences.java 237 2006-07-30 21:16:59Z olk $
 */
public class Preferences extends Properties {
    public static final String PROPERTYFILENAME = "weightmo.properties";
    public static final String PROTECTOLDVALUES = "protectOldValues";
    public static final String DATAFILE = "dataFile";
    public static final String DRAWPLAN = "drawPlan";
    public static final String DRAWPROJECTION = "drawProjection";
    public static final String SHOWABSTREND = "showAbsTrend";
    
    
    
    public void setDefaults() {
        setDataFile("weightmon.data");
        setDrawPlan(false);
        setDrawProjection(false);
        setProtectOldValues(true);
    }
    
    public boolean getBoolProperty(String key) {
        return getProperty(key) != null? getProperty(key).equals("true"):false;
    }
    public void setBoolProperty(String key, boolean value) {
        setProperty(key, ""+value);
    }
    
    /**
     * @return the dataFile
     */
    public String getDataFile() {
        return getProperty(DATAFILE);
    }
    /**
     * @param dataFile the dataFile to set
     */
    public void setDataFile(String dataFile) {
        setProperty(DATAFILE, dataFile);
    }
    /**
     * @return the drawPlan
     */
    public boolean getDrawPlan() {
        return getBoolProperty(DRAWPLAN);
    }
    /**
     * @param drawPlan the drawPlan to set
     */
    public void setDrawPlan(boolean drawPlan) {
        setBoolProperty(DRAWPLAN, drawPlan);
    }
    /**
     * @return the drawProjection
     */
    public boolean getDrawProjection() {
        return getBoolProperty(DRAWPROJECTION);
    }
    /**
     * @param drawProjection the drawProjection to set
     */
    public void setDrawProjection(boolean drawProjection) {
        setBoolProperty(DRAWPROJECTION, drawProjection);
    }
    /**
     * @return the protectOldValues
     */
    public boolean getProtectOldValues() {
        return getBoolProperty(PROTECTOLDVALUES);
    }
    /**
     * @param protectOldValues the protectOldValues to set
     */
    public void setProtectOldValues(boolean protectOldValues) {
        setBoolProperty(PROTECTOLDVALUES, protectOldValues);
    }

    public boolean getShowAbsTrend() {
        return getBoolProperty(SHOWABSTREND);
    }
    /**
     * @param protectOldValues the protectOldValues to set
     */
    public void setShowAbsTrend(boolean showAbsTrend) {
        setBoolProperty(SHOWABSTREND, showAbsTrend);
    }
    
    public void load() {
        File f = new File(PROPERTYFILENAME);
        InputStream inStream = null;
        try {
            inStream = new FileInputStream(f);
            load(inStream);
        } catch (FileNotFoundException e) {
            setDefaults();
        } catch (IOException e) {
            setDefaults();
        } finally {
            if (inStream != null) {
                try {
                    inStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }
    
    public void save() {
        File f = new File(PROPERTYFILENAME);
        OutputStream outStream = null;
        try {
            outStream = new FileOutputStream(f);
            store(outStream, "properties of the weightmonitor");
        } catch (FileNotFoundException e) {
            setDefaults();
        } catch (IOException e) {
            setDefaults();
        } finally {
            if (outStream != null) {
                try {
                    outStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }
    
}
