/*
 *  Copyright (c) 2006  Peter Kliem (Peter.Kliem@jaret.de)
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 */
package de.jaret.ppc.wm.model;

import java.text.DateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.StringTokenizer;

import de.jaret.ppc.wm.Util;
/**
 * Entry for one day.
 * 
 * @author kliem
 * @version $Id: WeightEntry.java 237 2006-07-30 21:16:59Z olk $
 */
public class WeightEntry {
    private Date _date;
    private double _weight;
    private double _avgWeigth;
    private boolean _mark = false;
    private String _comment;
    
    private static DateFormat _dateFormat = DateFormat.getDateInstance(); 
    private static Calendar _calendar = GregorianCalendar.getInstance();
    
    public WeightEntry(Date date) {
        _date = date;
    }

    public WeightEntry(int year, int month, int day) {
        Date d = Util.createDate(year, month, day);
        _date = d;
    }
    
    public WeightEntry(String line) {
        StringTokenizer tokenizer = new StringTokenizer(line, ";");
        String dateString = tokenizer.nextToken();
        try {
            _date = _dateFormat.parse(dateString);
        } catch (Exception e) {
            throw new RuntimeException("illegal Date "+dateString);
        }
        String weightString = tokenizer.nextToken();
        _weight = Double.parseDouble(weightString);
        String markString = tokenizer.nextToken();
        _mark = markString.equals("true");
        
        int position = dateString.length()+weightString.length()+markString.length()+4;
        String comment = line.substring(position, line.length()-1);
        if (comment.length() == 0) {
            _comment = null;
        } else {
            _comment = comment;
        }
        
    }
    
    public String toLine() {
        if (_date == null) {
            throw new RuntimeException("Date should not be null");
        }
        StringBuffer buf = new StringBuffer();
        buf.append(_dateFormat.format(_date));
        buf.append(";");
        buf.append(_weight);
        buf.append(";");
        buf.append(_mark);
        buf.append(";");
        buf.append(_comment != null?"\""+_comment+"\"":"\"\"");
        return buf.toString();
    }
    
    /**
     * @return the avgWeigth
     */
    public double getAvgWeigth() {
        return _avgWeigth;
    }
    /**
     * @param avgWeigth the avgWeigth to set
     */
    public void setAvgWeigth(double avgWeigth) {
        _avgWeigth = avgWeigth;
    }
    /**
     * @return the date
     */
    public Date getDate() {
        return _date;
    }
    /**
     * @param date the date to set
     */
    public void setDate(Date date) {
        _date = date;
    }
    /**
     * @return the weight
     */
    public double getWeight() {
        return _weight;
    }
    /**
     * @param weight the weight to set
     */
    public void setWeight(double weight) {
        _weight = weight;
    }

    /**
     * @return the comment
     */
    public String getComment() {
        return _comment;
    }

    /**
     * @param comment the comment to set
     */
    public void setComment(String comment) {
        _comment = comment;
    }

    /**
     * @return the mark
     */
    public boolean isMark() {
        return _mark;
    }

    /**
     * @param mark the mark to set
     */
    public void setMark(boolean mark) {
        _mark = mark;
    }

    public Integer getYear() {
        _calendar.setTime(_date);
        return new Integer(_calendar.get(Calendar.YEAR));
    }
    public Integer getMonth() {
        _calendar.setTime(_date);
        return new Integer(_calendar.get(Calendar.MONTH));
    }
    public Integer getDay() {
        _calendar.setTime(_date);
        return new Integer(_calendar.get(Calendar.DAY_OF_MONTH));
    }
    
    
}
