/*
 *  Copyright (c) 2006  Peter Kliem (Peter.Kliem@jaret.de)
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 */
package de.jaret.ppc.wm.ui;

import java.text.DateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;

import de.jaret.ppc.wm.Util;
import de.jaret.ppc.wm.WeightMonitor;
import de.jaret.ppc.wm.model.WeightEntry;

/**
 * Dialog showing the weight graph.
 * 
 * @author kliem
 * @version $Id: GraphDialog.java 237 2006-07-30 21:16:59Z olk $
 */
public class GraphDialog extends Dialog {
    public static final int MODE_LAST30 = 0;
    public static final int MODE_MONTH = 1;
    public static final int MODE_6MONTH = 2;
    public static final int MODE_YEAR = 3;


    private static final DateFormat _df = DateFormat.getDateInstance();
    private int _year;
    private int _month;
    private int _day;

    private int _mode = MODE_LAST30;
    private static Calendar _calendar = GregorianCalendar.getInstance();
    List _entries;

    private Graph _graph;
    private Label _headlineLabel;

    public GraphDialog(Shell parent, int style) {
        super(parent, style);
    }

    public GraphDialog(Shell parent) {
        this(parent, 0);
    }

    /**
     * Show the dialog.
     * @return <code>null</code>
     */
    public Object open() {
        splitDate(new Date());

        Shell parent = getParent();
        final Shell shell = new Shell(parent, SWT.PRIMARY_MODAL | SWT.RESIZE);
        shell.setText("Graph");

        Menu mb = new Menu(shell, SWT.BAR);
        shell.setMenuBar(mb);

        MenuItem okItem = new MenuItem(mb, SWT.PUSH);
        okItem.setText("OK");
        okItem.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                shell.dispose();
            }
        });

        createContents(shell);

        shell.open();
        Display display = parent.getDisplay();
        while (!shell.isDisposed()) {
            if (!display.readAndDispatch())
                display.sleep();
        }
        return null;
    }

    /**
     * Split a date to year/month/day.
     * 
     * @param date Date to split
     */
    private void splitDate(Date date) {
        _year = _calendar.get(Calendar.YEAR);
        _month = _calendar.get(Calendar.MONTH);
        _day = _calendar.get(Calendar.DAY_OF_MONTH);
    }

    /**
     * Create the widgets.
     * @param shell parent shell
     */
    private void createContents(Shell shell) {
        // Default Last30
        _entries = WeightMonitor.getInstance().getWeightList().getEntries(new Date(), 30);

        shell.setLayout(new GridLayout(3, false));

        Button dec = new Button(shell, SWT.ARROW | SWT.LEFT);
        GridData gd = new GridData(SWT.LEFT, SWT.BOTTOM, false, false);
        dec.setLayoutData(gd);
        dec.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                decrement();
            }
        });
        _headlineLabel = new Label(shell, SWT.CENTER);
        gd = new GridData(SWT.CENTER, SWT.BOTTOM, true, false);
        gd = new GridData(GridData.FILL_HORIZONTAL);
        _headlineLabel.setLayoutData(gd);
        updateHeadline();

        Button inc = new Button(shell, SWT.ARROW | SWT.RIGHT);
        gd = new GridData(SWT.RIGHT, SWT.BOTTOM, false, false);
        inc.setLayoutData(gd);
        inc.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                increment();
            }
        });


        _graph = new Graph(shell, SWT.NULL);
        gd = new GridData(GridData.FILL_BOTH);
        gd.horizontalSpan = 3;
        _graph.setLayoutData(gd);
        _graph.setData(_entries);

        Composite buttonPanel = new Composite(shell, SWT.NONE);
        gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 3;
        buttonPanel.setLayoutData(gd);
        buttonPanel.setLayout(new RowLayout(SWT.HORIZONTAL));

        Button b = new Button(buttonPanel, SWT.PUSH);
        b.setText("Year");
        b.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                _entries = WeightMonitor.getInstance().getWeightList().getEntriesFirst(Util.createDate(_year, 0, 1), 365);
                updateHeadline();
                _graph.setDrawMark(false);
                _graph.setData(_entries);
                _mode = MODE_YEAR;
            }
        });

        b = new Button(buttonPanel, SWT.PUSH);
        b.setText("6 Month");
        b.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                _entries = WeightMonitor.getInstance().getWeightList().getEntriesFirst(Util.createDate(_year, _month, 1), 182);
                updateHeadline();
                _graph.setDrawMark(false);
                _graph.setData(_entries);
                _mode = MODE_6MONTH;
            }
        });

        b = new Button(buttonPanel, SWT.PUSH);
        b.setText("Month");
        b.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                _entries = WeightMonitor.getInstance().getWeightList().getEntriesFirst(Util.createDate(_year, _month, 1), 31);
                updateHeadline();
                _graph.setDrawMark(true);
                _graph.setData(_entries);
                _mode = MODE_MONTH;
            }
        });

        b = new Button(buttonPanel, SWT.PUSH);
        b.setText("Last30");
        b.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                _entries = WeightMonitor.getInstance().getWeightList().getEntries(new Date(), 31);
                updateHeadline();
                _graph.setDrawMark(true);
                _graph.setData(_entries);
                _mode = MODE_LAST30;
            }
        });


    }

    /**
     * Update the text of the headline label.
     *
     */
    private void updateHeadline() {
        Date begin = ((WeightEntry) _entries.get(0)).getDate();
        Date end = ((WeightEntry) _entries.get(_entries.size() - 1)).getDate();
        _headlineLabel.setText(_df.format(begin) + " - " + _df.format(end));
    }

    /**
     * Handle decrement button.
     *
     */
    private void decrement() {
        switch (_mode) {
        case MODE_LAST30:
            _month -= 1;
            if (_month < 0) {
                _month = 11;
                _year -= 1;
            }
            _entries = WeightMonitor.getInstance().getWeightList().getEntries(Util.createDate(_year, _month, _day), 30);
            break;
        case MODE_MONTH:
            _month -= 1;
            if (_month < 0) {
                _month = 11;
                _year -= 1;
            }
            _entries = WeightMonitor.getInstance().getWeightList().getEntriesFirst(Util.createDate(_year, _month, 1), 31);
            break;
        case MODE_6MONTH:
            _month -= 6;
            if (_month < 0) {
                _month = _month % 12;
                _year -= 1;
            }
            _entries = WeightMonitor.getInstance().getWeightList().getEntriesFirst(Util.createDate(_year, _month, 1), 182);
            break;
        case MODE_YEAR:
            _year -= 1;
            _entries = WeightMonitor.getInstance().getWeightList().getEntriesFirst(Util.createDate(_year, 0, 1), 365);
            break;

        default:
            throw new RuntimeException("illegal mode");
        }
        updateHeadline();
        _graph.setData(_entries);
    }

    /**
     * Handle increment button.
     *
     */
    private void increment() {
        switch (_mode) {
        case MODE_LAST30:
            _month += 1;
            if (_month > 11) {
                _month = 0;
                _year += 1;
            }
            _entries = WeightMonitor.getInstance().getWeightList().getEntries(Util.createDate(_year, _month, _day), 30);
            break;
        case MODE_MONTH:
            _month += 1;
            if (_month > 11) {
                _month = 0;
                _year += 1;
            }
            _entries = WeightMonitor.getInstance().getWeightList().getEntriesFirst(Util.createDate(_year, _month, 1), 31);
            break;
        case MODE_6MONTH:
            _month += 6;
            if (_month > 11) {
                _month = _month % 12;
                _year += 1;
            }
            _entries = WeightMonitor.getInstance().getWeightList().getEntriesFirst(Util.createDate(_year, _month, 1), 182);
            break;
        case MODE_YEAR:
            _year += 1;
            _entries = WeightMonitor.getInstance().getWeightList().getEntriesFirst(Util.createDate(_year, 0, 1), 365);
            break;

        default:
            throw new RuntimeException("illegal mode");
        }
        updateHeadline();
        _graph.setData(_entries);
    }
}