/*
 *  Copyright (c) 2006  Peter Kliem (Peter.Kliem@jaret.de)
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 */

package de.jaret.ppc.wm.ui;

import java.text.NumberFormat;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

import de.jaret.ppc.wm.WeightMonitor;
import de.jaret.ppc.wm.model.Preferences;
import de.jaret.ppc.wm.model.WeightList;

/**
 * The main class for the WeightMonitor. This class controls the initial gui including event loop, loading of the data 
 * (after gui was displayed) and some global stuff.
 * 
 * @author kliem
 * @version $Id: Main.java 237 2006-07-30 21:16:59Z olk $
 */
public class Main {
    public static final String VERSION = "0.1";
    public static final String RELEASEDATE = "20060731";
    private static Color _red;
    private static Color _green;

    private static final NumberFormat _nf = NumberFormat.getNumberInstance();
    static {
        _nf.setMaximumFractionDigits(2);
        _nf.setMinimumFractionDigits(2);
    }

    private int _days[] = { 7, 14, 30, 60 };
    private Label _trendLabels[] = new Label[_days.length];
    private Label _absLabels[] = new Label[_days.length];
    private Label _dayLabels[] = new Label[_days.length];
    private Label _calLabels[] = new Label[_days.length];

    private Shell _shell;

    public void open() {
        final Display display = new Display();

        /* 
         * Create a Shell with the default style
         * i.e. full screen, no decoration on PocketPC.
         * Alternative: 'new Shell(display, SWT.CLOSE)'
         * to get the Pocket PC 'Ok' button.
         */
        _shell = new Shell(display, SWT.RESIZE);

        _shell.addShellListener(new ShellAdapter() {

            public void shellActivated(ShellEvent e) {
                update();
            }

            public void shellClosed(ShellEvent e) {
                // save preferences
                WeightMonitor.getInstance().getPreferences().save();
                // save values
                String error = WeightMonitor.getInstance().getWeightList().save();
                if (error != null) {
                    MessageBox mb = new MessageBox(_shell, SWT.ICON_ERROR);
                    mb.setMessage(error);
                    mb.setText("Error");
                    mb.open();
                }
            }

        });


        /* 
         * Set a text so that the top level Shell
         * also appears in the Pocket PC task list
         */
        _shell.setText("WeightMonitor");

        /*
         * Set a menubar to follow UI guidelines
         * on Pocket PC
         */
        Menu mb = new Menu(_shell, SWT.BAR);
        _shell.setMenuBar(mb);

        //////// dialog menu

        final Menu dialogMenu = new Menu(_shell, SWT.POP_UP);
        MenuItem item = new MenuItem(dialogMenu, SWT.CASCADE);
        item.setText("Month");
        item.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                showMonthDialog(_shell);
            }
        });
        item = new MenuItem(dialogMenu, SWT.CASCADE);
        item.setText("Graph");
        item.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                showGraphDialog(_shell);
            }
        });
        item = new MenuItem(dialogMenu, SWT.CASCADE);
        item.setText("Preferences");
        item.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                showPreferenceDialog(_shell);
            }
        });

        MenuItem dialogMenuTop = new MenuItem(mb, SWT.PUSH);
        dialogMenuTop.setText("Dialog");
        dialogMenuTop.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                dialogMenu.setVisible(true);
            }
        });

        //////// file menu

        final Menu fileMenu = new Menu(_shell, SWT.POP_UP);
        item = new MenuItem(fileMenu, SWT.CASCADE);
        item.setText("About");
        item.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                showAboutDialog(_shell);
            }
        });
        item = new MenuItem(fileMenu, SWT.CASCADE);
        item.setText("Exit");
        item.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                _shell.close();
            }
        });

        MenuItem fileMenuTop = new MenuItem(mb, SWT.PUSH);
        fileMenuTop.setText("File");
        fileMenuTop.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                fileMenu.setVisible(true);
            }
        });

        createContents(_shell);

        load(_shell);

        _red = Display.getCurrent().getSystemColor(SWT.COLOR_RED);
        _green = Display.getCurrent().getSystemColor(SWT.COLOR_DARK_GREEN);

        _shell.open();

        // event loop
        while (!_shell.isDisposed()) {
            if (!display.readAndDispatch())
                display.sleep();
        }
        display.dispose();
    }

    private void createContents(Shell shell) {
        shell.setLayout(new GridLayout(5, false));

        Label headlineLabel = new Label(shell, SWT.CENTER);
        GridData gd = new GridData(SWT.CENTER, SWT.BOTTOM, true, false);
        gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 5;
        headlineLabel.setLayoutData(gd);
        headlineLabel.setText("WeightMonitor");

        Label l = new Label(shell, SWT.NONE);
        gd = new GridData();
        l.setLayoutData(gd);
        l.setText("Interval");

        l = new Label(shell, SWT.NONE);
        gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
        l.setLayoutData(gd);
        l.setText("Abs");

        l = new Label(shell, SWT.NONE);
        gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
        l.setLayoutData(gd);
        l.setText("kg/day");

        l = new Label(shell, SWT.NONE);
        gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
        l.setLayoutData(gd);
        l.setText("kCal/day");

        l = new Label(shell, SWT.NONE);
        gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
        l.setLayoutData(gd);
        l.setText("");

        for (int i = 0; i < _days.length; i++) {
            l = new Label(shell, SWT.NONE);
            gd = new GridData();
            l.setLayoutData(gd);
            _dayLabels[i] = l;

            l = new Label(shell, SWT.NONE);
            gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
            l.setLayoutData(gd);
            _absLabels[i] = l;

            l = new Label(shell, SWT.NONE);
            gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
            l.setLayoutData(gd);
            _trendLabels[i] = l;

            l = new Label(shell, SWT.NONE);
            gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
            l.setLayoutData(gd);
            _calLabels[i] = l;

            // push everything to the left
            l = new Label(shell, SWT.NONE);
            gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
            l.setLayoutData(gd);
            l.setText("");
        }
    }

    private void update() {
        WeightList wl = WeightMonitor.getInstance().getWeightList();
        WeightList.Trend t = null;

        for (int i = 0; i < _days.length; i++) {
            int days = _days[i];
            _dayLabels[i].setText("Last " + days + " days:");
            t = wl.getTrend(_days[i]);
            if (t.valid) {
                Color fg = t.kgPerDay <= 0 ? _green : _red;
                _absLabels[i].setText(_nf.format(t.kgAbs));
                _trendLabels[i].setForeground(fg);
                _trendLabels[i].setText(_nf.format(t.kgPerDay));
                _trendLabels[i].redraw();
                double calPerDay = t.kgPerDay * 7300;
                _calLabels[i].setText(_nf.format(calPerDay));
            } else {
                _absLabels[i].setText("--");
                _trendLabels[i].setText("--");
                _calLabels[i].setText("--");
            }
        }
    }

    private void load(final Shell shell) {

        Runnable r = new Runnable() {
            public void run() {
                // load preferences
                Preferences prefs = WeightMonitor.getInstance().getPreferences();
                prefs.load();
                // load values
                String error = WeightMonitor.getInstance().getWeightList().load();
                if (error != null) {
                    MessageBox mb = new MessageBox(shell, SWT.ICON_ERROR);
                    mb.setMessage(error);
                    mb.setText("Error");
                    mb.open();
                }
            }
        };
        r.run();

    }

    private void showAboutDialog(Shell shell) {
        MessageBox dialog = new MessageBox(shell);//, SWT.ICON_INFORMATION|SWT.OK);
        dialog.setText("About");
        String swtVersion = "" + SWT.getVersion();
        dialog.setMessage("WeightMonitor v" + VERSION + "\n(c) 2006 by Peter Kliem\nLicense: EPL, see files in distribution\n\n" + "Releasedate " + RELEASEDATE
                + "\nSWT version:" + swtVersion);
        dialog.open();
    }

    private void showGraphDialog(Shell shell) {
        GraphDialog dialog = new GraphDialog(shell);
        dialog.open();
    }

    private void showMonthDialog(Shell shell) {
        MonthDialog dialog = new MonthDialog(shell);
        dialog.open(2006, 6);
    }

    private void showPreferenceDialog(Shell shell) {
        PreferenceDialog dialog = new PreferenceDialog(shell);
        dialog.open();
    }

}