/*
 *  Copyright (c) 2006  Peter Kliem (Peter.Kliem@jaret.de)
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 */
package de.jaret.ppc.wm.ui;

import java.text.DateFormatSymbols;
import java.text.NumberFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import de.jaret.ppc.ui.DoubleField;
import de.jaret.ppc.ui.ScrolledComposite;
import de.jaret.ppc.wm.WeightMonitor;
import de.jaret.ppc.wm.model.WeightEntry;

/**
 * Dialog showing the input fields for one month.
 * 
 * @author kliem
 * @version $Id: MonthDialog.java 239 2006-07-31 21:04:23Z olk $
 */
public class MonthDialog extends Dialog implements FocusListener, SelectionListener {
    private static final boolean _showAbs = WeightMonitor.getInstance().getPreferences().getShowAbsTrend();
    private static final DateFormatSymbols _dateFormatSymbols = new DateFormatSymbols();
    private static final NumberFormat _nf = NumberFormat.getNumberInstance();

    private int _year;
    private int _month;
    private int _day;

    private static final Color _red = Display.getCurrent().getSystemColor(SWT.COLOR_RED);
    private static final Color _green = Display.getCurrent().getSystemColor(SWT.COLOR_DARK_GREEN);
    private static final Color _black = Display.getCurrent().getSystemColor(SWT.COLOR_BLACK);
    private ScrolledComposite _scrolledComposite;

    private Label[] _dayLabels = new Label[31];
    private Label[] _dayOfWeekLabels = new Label[31];
    private DoubleField[] _weightFields = new DoubleField[31];
    private Label[] _avgLabels = new Label[31];
    private Button[] _markChecks = new Button[31];
    private WeightEntry[] _weightEntries = new WeightEntry[31];
    private Text[] _commentTexts = new Text[31];

    public MonthDialog(Shell parent, int style) {
        super(parent, style);
        _nf.setMaximumFractionDigits(1);
        _nf.setMinimumFractionDigits(1);
    }

    public MonthDialog(Shell parent) {
        this(parent, 0); // your default style bits go here (not the Shell's style bits)
    }

    /**
     * Show the dialog.
     * @param year
     * @param month rage is 0-11
     * @return <code>null</code>
     */
    public Object open(int year, int month) {
        _year = year;
        _month = month;

        Shell parent = getParent();
        final Shell shell = new Shell(parent, SWT.PRIMARY_MODAL | SWT.RESIZE);

        // Your code goes here (widget creation, set result, etc).

        Menu mb = new Menu(shell, SWT.BAR);
        shell.setMenuBar(mb);

        MenuItem okItem = new MenuItem(mb, SWT.PUSH);
        okItem.setText("OK");
        okItem.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                shell.forceFocus(); // ensures, that the current value will be stored when focus moves
                shell.dispose();
            }
        });

        createContents(shell);

        shell.open();
        
        Display display = parent.getDisplay();
        while (!shell.isDisposed()) {
            if (!display.readAndDispatch())
                display.sleep();
        }
        return null;
    }

    Label _headlineLabel;

    private void createContents(Composite parent) {
        parent.setLayout(new GridLayout(3, false));

        Button dec = new Button(parent, SWT.ARROW | SWT.LEFT);
        GridData gd = new GridData(SWT.LEFT, SWT.BOTTOM, false, false);
        dec.setLayoutData(gd);
        dec.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                decrement();
            }
        });
        _headlineLabel = new Label(parent, SWT.CENTER);
        gd = new GridData(GridData.FILL_HORIZONTAL);
        _headlineLabel.setLayoutData(gd);
        updateHeadline();

        Button inc = new Button(parent, SWT.ARROW | SWT.RIGHT);
        gd = new GridData(SWT.RIGHT, SWT.BOTTOM, false, false);
        inc.setLayoutData(gd);
        inc.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                increment();
            }
        });


        _scrolledComposite = new ScrolledComposite(parent, SWT.H_SCROLL | SWT.V_SCROLL);
        gd = new GridData(GridData.FILL_BOTH);
        gd.horizontalSpan = 3;
        _scrolledComposite.setLayoutData(gd);

        _scrolledComposite.addControlListener(new ControlAdapter() {
            public void controlResized(ControlEvent e) {
                Control c = e.display.getFocusControl();
                if (c instanceof Text && _day >0) {
                    // does not work with eSWT -- really don't know why
                    scrollToVisible((Text)c);
                    scrollToVisible(_weightFields[_day-1].getText());
                }
            }
        });
        
        Composite panel = new Composite(_scrolledComposite, SWT.NONE);
        _scrolledComposite.setContent(panel);
        GridLayout gl = new GridLayout(6, false);
        panel.setLayout(gl);


        int digitWidth = 8;

        for (int i = 1; i <= 31; i++) {
            Label day = new Label(panel, SWT.RIGHT);
            _dayLabels[i - 1] = day;
            gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
            day.setLayoutData(gd);

            Label dayOfWeek = new Label(panel, SWT.LEFT);
            _dayOfWeekLabels[i - 1] = dayOfWeek;

            Text weightText = new Text(panel, SWT.BORDER);
            weightText.setData("day", new Integer(i));
            weightText.addFocusListener(this);
            weightText.setTextLimit(5);
            GridData weightGD = new GridData();
            weightGD.widthHint = digitWidth * weightText.getTextLimit();
            weightText.setLayoutData(weightGD);
            DoubleField df = new DoubleField(1, 0, 300);
            df.setText(weightText);
            _weightFields[i - 1] = df;

            Label avgText = new Label(panel, SWT.NULL);
            GridData avgGD = new GridData();
            avgGD.widthHint = digitWidth * 5;
            avgText.setLayoutData(avgGD);
            _avgLabels[i - 1] = avgText;

            Button markCheck = new Button(panel, SWT.CHECK);
            _markChecks[i - 1] = markCheck;
            markCheck.addSelectionListener(this);
            markCheck.setData("day", new Integer(i));

            Text commentText = new Text(panel, SWT.BORDER);
            commentText.setTextLimit(40);
            gd = new GridData(200, SWT.DEFAULT);
            _commentTexts[i - 1] = commentText;
            commentText.setData("day", new Integer(i));
            commentText.setData("comment", "x");
            commentText.addFocusListener(this);
            commentText.setLayoutData(gd);
        }
        populateFields();

        panel.setSize(panel.computeSize(SWT.DEFAULT, SWT.DEFAULT));

        Calendar cal = GregorianCalendar.getInstance();
        int d = cal.get(Calendar.DAY_OF_MONTH);
        Text field = _weightFields[d - 1].getText();
        field.forceFocus();
    }

    private void populateFields() {
        boolean protectOldValues = WeightMonitor.getInstance().getPreferences().getProtectOldValues();
        Calendar calendar = GregorianCalendar.getInstance();
        calendar.set(Calendar.YEAR, _year);
        calendar.set(Calendar.MONTH, _month);
        calendar.set(Calendar.DAY_OF_MONTH, 1);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        
        int numDays = calendar.getActualMaximum(Calendar.DAY_OF_MONTH);
        Date today = new Date();
        
        for (int i = 1; i <= 31; i++) {
            boolean protect = protectOldValues && today.compareTo(calendar.getTime()) >= 0; 
            WeightEntry we = null;
            if (i <= numDays) {
                we = WeightMonitor.getInstance().getWeightList().getEntry(_year, _month, i);
                _weightEntries[i - 1] = we;
            } else {
                _weightEntries[i - 1] = null;
            }

            Label day = _dayLabels[i - 1];
            if (i <= numDays) {
                day.setText(Integer.toString(i));
            } else {
                day.setText("-");
            }

            Label dayOfWeek = _dayOfWeekLabels[i - 1];
            if (i <= numDays) {
                int dow = calendar.get(Calendar.DAY_OF_WEEK);
                dayOfWeek.setText(_dateFormatSymbols.getShortWeekdays()[dow]);
                if (dow == Calendar.SUNDAY) {
                    dayOfWeek.setForeground(_red);
                } else {
                    dayOfWeek.setForeground(_black);
                }
            } else {
                dayOfWeek.setForeground(_black);
                dayOfWeek.setText("-");
            }

            Text weightText = _weightFields[i - 1].getText();
            DoubleField df = _weightFields[i - 1];
            if (we != null) {
                df.setValue(we.getWeight());
            } else {
                weightText.setText("");
            }
            if (i <= numDays && !protect) {
                weightText.setEnabled(true);
            } else {
                weightText.setEnabled(false);
            }

            updateTrend(i);

            Button markCheck = _markChecks[i - 1];
            if (we != null) {
                markCheck.setSelection(we.isMark());
                markCheck.setEnabled(!protect);
            } else {
                markCheck.setSelection(false);
                markCheck.setEnabled(false);
            }
            
            Text commentText = _commentTexts[i-1];
            if (we != null) {
                commentText.setText(we.getComment()!=null?we.getComment():"");
                commentText.setEnabled(!protect);
            } else {
                commentText.setEnabled(false);
                commentText.setText("");
            }
            
            calendar.add(Calendar.DAY_OF_YEAR, 1);
        }
    }


    private void updateHeadline() {
        String monthName = new DateFormatSymbols().getMonths()[_month];
        _headlineLabel.setText(monthName + " " + _year);
        _headlineLabel.redraw();
    }

    private void decrement() {
        _month -= 1;
        if (_month < 0) {
            _month = 11;
            _year -= 1;
        }
        populateFields();
        updateHeadline();
    }

    private void increment() {
        _month += 1;
        if (_month > 11) {
            _month = 0;
            _year += 1;
        }
        populateFields();
        updateHeadline();
    }

    private void updateTrend(int i) {
        WeightEntry we = _weightEntries[i - 1];
        Label avgLabel = _avgLabels[i - 1];
        if (we != null) {
            double dif = we.getWeight() - we.getAvgWeigth();
            if (_showAbs) {
                avgLabel.setText(_nf.format(we.getAvgWeigth()));
            } else {
                avgLabel.setText(_nf.format(dif));
            }
            if (dif < 0) {
                avgLabel.setForeground(_green);
            } else {
                avgLabel.setForeground(_red);
            }
        } else {
            avgLabel.setForeground(_black);
            avgLabel.setText("-");
        }
    }


    private void scrollToVisible(Text text) {
        Point pos = text.getLocation();
        pos.y += text.getSize().y;
        pos.y = pos.y - _scrolledComposite.getSize().y;
        
        if (_scrolledComposite.getHorizontalBar() != null) {
            pos.y += _scrolledComposite.getHorizontalBar().getSize().y;
        }
        if (pos.y > 0) {
            _scrolledComposite.setOrigin(0, pos.y);
        }
    }

    public void focusGained(FocusEvent e) {
        scrollToVisible((Text) e.widget);
        _day = ((Integer)((Text) e.widget).getData("day")).intValue();
    }

    public void focusLost(FocusEvent e) {
        int day = ((Integer) e.widget.getData("day")).intValue();
        if (e.widget.getData("comment") == null) {
            double input = _weightFields[day - 1].getValue();
            if (input != 0.0) {
                WeightEntry we = _weightEntries[day - 1];
                if (we == null) {
                    we = new WeightEntry(_year, _month, day);
                    WeightMonitor.getInstance().getWeightList().registerEntry(we);
                    _weightEntries[day - 1] = we;
                    _markChecks[day - 1].setEnabled(true);
                    _commentTexts[day -1].setEnabled(true);
                }
                we.setWeight(input);
                WeightMonitor.getInstance().getWeightList().updateTrend(we);
                updateTrend(day);
            }
        } else {
            // copmment field
            WeightEntry we = _weightEntries[day - 1];
            if (we != null) {
                String input = _commentTexts[day-1].getText();
                we.setComment(input);
            }
        }
    }

    public void widgetDefaultSelected(SelectionEvent e) {
    }

    public void widgetSelected(SelectionEvent e) {
        int day = ((Integer) e.widget.getData("day")).intValue();
        WeightEntry we = _weightEntries[day - 1];
        if (we != null) {
            we.setMark(_markChecks[day - 1].getSelection());
        }
    }
}