/*
 *  Copyright (c) 2006  Peter Kliem (Peter.Kliem@jaret.de)
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 */
package de.jaret.ppc.wm.ui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;

import de.jaret.ppc.ui.ScrolledComposite;
import de.jaret.ppc.wm.WeightMonitor;
import de.jaret.ppc.wm.model.Preferences;
/**
 * Dialog for editing the preferences.
 * 
 * @author kliem
 * @version $Id: PreferenceDialog.java 237 2006-07-30 21:16:59Z olk $
 */
public class PreferenceDialog extends Dialog {
    ScrolledComposite _scrolledComposite;
    Preferences _prefs;
    Button _protectOldValues;
    Button _drawPlan;
    Button _absTrendValues;
    Button _drawProjection;
    Button _showAbsTrend;
    
    public PreferenceDialog(Shell parent, int style) {
        super(parent, style);
    }

    public PreferenceDialog(Shell parent) {
        this(parent, 0); // your default style bits go here (not the Shell's style bits)
    }

    public Object open() {
        _prefs = WeightMonitor.getInstance().getPreferences();
        
        Shell parent = getParent();
        final Shell shell = new Shell(parent, SWT.PRIMARY_MODAL | SWT.RESIZE);


        Menu mb = new Menu(shell, SWT.BAR);
        shell.setMenuBar(mb);


        MenuItem okItem = new MenuItem(mb, SWT.PUSH);
        okItem.setText("OK");
        okItem.addSelectionListener(new SelectionListener() {

            public void widgetDefaultSelected(SelectionEvent e) {
            }

            public void widgetSelected(SelectionEvent e) {
                storeValues();
                shell.dispose();
            }


        });

        createContents(shell);

        shell.open();
        Display display = parent.getDisplay();
        while (!shell.isDisposed()) {
            if (!display.readAndDispatch())
                display.sleep();
        }
        return null;
    }

    private void storeValues() {
        _prefs.setDrawPlan(_drawPlan.getSelection());
        _prefs.setDrawProjection(_drawProjection.getSelection());
        _prefs.setProtectOldValues(_protectOldValues.getSelection());
        _prefs.setShowAbsTrend(_showAbsTrend.getSelection());
    }

    private void createContents(Composite parent) {
        parent.setLayout(new GridLayout(1, false));

        Label titleLabel = new Label(parent, SWT.CENTER);
        titleLabel.setText("Preferences");
        GridData tlgd = new GridData(SWT.CENTER, SWT.BOTTOM, true, false);
        titleLabel.setLayoutData(tlgd);

        _scrolledComposite = new ScrolledComposite(parent, SWT.H_SCROLL | SWT.V_SCROLL);
        GridData scgd = new GridData(GridData.FILL_BOTH);
        _scrolledComposite.setLayoutData(scgd);

        Composite panel = new Composite(_scrolledComposite, SWT.NONE);
        _scrolledComposite.setContent(panel);
        GridLayout gl = new GridLayout(2, false);
        panel.setLayout(gl);

        Label l = new Label(panel, SWT.NONE);
        l.setText("DataFile:");
        GridData gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
        l.setLayoutData(gd);
        l = new Label(panel, SWT.NONE);
        l.setText(_prefs.getDataFile());
        
        l = new Label(panel, SWT.NONE);
        l.setText("Protect values");
        gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
        l.setLayoutData(gd);
        _protectOldValues = new Button(panel, SWT.CHECK);
        _protectOldValues.setSelection(_prefs.getProtectOldValues());

        l = new Label(panel, SWT.NONE);
        l.setText("Draw plan");
        gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
        l.setLayoutData(gd);
        _drawPlan = new Button(panel, SWT.CHECK);
        _drawPlan.setSelection(_prefs.getDrawPlan());
        _drawPlan.setEnabled(false); // not yet implemented

        l = new Label(panel, SWT.NONE);
        l.setText("Draw projection");
        gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
        l.setLayoutData(gd);
        _drawProjection = new Button(panel, SWT.CHECK);
        _drawProjection.setSelection(_prefs.getDrawProjection());
        _drawProjection.setEnabled(false);//not yet implemented

        l = new Label(panel, SWT.NONE);
        l.setText("Absolute trend values");
        gd = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
        l.setLayoutData(gd);
        _showAbsTrend = new Button(panel, SWT.CHECK);
        _showAbsTrend.setSelection(_prefs.getShowAbsTrend());



        panel.setSize(panel.computeSize(SWT.DEFAULT, SWT.DEFAULT));
    }


}